@@ -23,13 +23,14 @@ require SQL::Statement::Util;
 
 use Carp qw(carp croak);
 use Clone qw(clone);
+use Errno;
 use Scalar::Util qw(blessed looks_like_number);
 use List::Util qw(first);
 use Params::Util qw(_INSTANCE _STRING _ARRAY _ARRAY0 _HASH0 _HASH);
 
 #use locale;
 
-$VERSION = '1.27';
+$VERSION = '1.28';
 
 sub new
 {
@@ -48,9 +49,14 @@ sub new
     $flags->{PrintError}    = 1 unless defined $flags->{PrintError};
     $flags->{text_numbers}  = 1 unless defined $flags->{text_numbers};
     $flags->{alpha_compare} = 1 unless defined $flags->{alpha_compare};
-    for ( keys %{$flags} )
+    
+    unless( blessed( $flags ) ) # avoid copying stale data from earlier parsing sessions
     {
-        $self->{$_} = $flags->{$_};
+        %$self = ( %$self, %{ clone( $flags ) } );
+    }
+    else
+    {
+	$self->{$_} = $flags->{$_} for qw(RaiseError PrintError opts);
     }
 
     $self->{dlm} = '~';
@@ -64,7 +70,8 @@ sub new
 
         $parser = SQL::Parser->new( $parser_dialect, $flags );
     }
-    $self->{termFactory} = SQL::Statement::TermFactory->new($self);
+    $self->{termFactory}  = SQL::Statement::TermFactory->new($self);
+    $self->{capabilities} = {};
     $self->prepare( $sql, $parser );
     return $self;
 }
@@ -130,6 +137,7 @@ sub execute
     my ($command) = $self->command();
     return $self->do_err('No command found!') unless ($command);
     ( $self->{NUM_OF_ROWS}, $self->{NUM_OF_FIELDS}, $self->{data} ) = $self->$command( $data, $params );
+    return unless( defined( $self->{NUM_OF_ROWS} ) );
 
     @{ $self->{NAME} } = map { $_->display_name() } @{ $self->{columns} };
 
@@ -140,7 +148,7 @@ sub execute
     {
         push( @{ $self->{tables} }, SQL::Statement::Table->new($_) );
     }
-    $self->{NUM_OF_ROWS} || '0E0';
+    return $self->{NUM_OF_ROWS} || '0E0';
 }
 
 sub CREATE ($$$)
@@ -170,11 +178,11 @@ sub CREATE ($$$)
             $names = $sth->{NAME};
         }
         $names = $sth->{NAME} unless defined $names;
-        my $tbl_data = $sth->{f_stmt}->{data};
-        my $tbl_name = $self->tables(0)->name;
+        my $tbl_data = $sth->{sql_stmt}->{data};
+        my $tbl_name = $self->{org_table_names}->[0] || $self->tables(0)->name;
 
-        # my @tbl_cols = map {$_->name} $sth->{f_stmt}->columns;
-        #my @tbl_cols=map{$_->name} $sth->{f_stmt}->columns if $sth->{f_stmt};
+        # my @tbl_cols = map {$_->name} $sth->{sql_stmt}->columns;
+        #my @tbl_cols=map{$_->name} $sth->{sql_stmt}->columns if $sth->{sql_stmt};
         my @tbl_cols;
 
         #            @tbl_cols=@{ $sth->{NAME} } unless @tbl_cols;
@@ -186,11 +194,12 @@ sub CREATE ($$$)
         $data->{Database}->do($create_sql);
         my $colstr     = ('?,') x @tbl_cols;
         my $insert_sql = "INSERT INTO $tbl_name VALUES($colstr)";
-        $data->{Database}->do( $insert_sql, {}, @$_ ) for @$tbl_data;
+        my $local_sth  = $data->{Database}->prepare($insert_sql);
+        $local_sth->execute(@$_) for @$tbl_data;
         return ( 0, 0 );
     }
     my ( $eval, $foo ) = $self->open_tables( $data, 1, 1 );
-    return undef unless ($eval);
+    return unless ($eval);
     $eval->params($params);
     my ( $row, $table, $col ) = ( [], $eval->table( $self->tables(0)->name() ) );
     if ( _ARRAY( $table->col_names() ) )
@@ -217,18 +226,25 @@ sub CALL
     ( $self->{NUM_OF_ROWS}, $self->{NUM_OF_FIELDS}, $self->{data} ) = $procTerm->value($data);
 }
 
+my $enoentstr = "Cannot open .*\(" . Errno::ENOENT . "\)";
+my $enoentrx  = qr/$enoentstr/;
+
 sub DROP ($$$)
 {
     my ( $self, $data, $params ) = @_;
-    if ( $self->{ignore_missing_table} )
+    my $eval;
+    my @err;
+    eval {
+        local $SIG{__WARN__} = sub { push @err, @_ };
+        ($eval) = $self->open_tables( $data, 0, 1 );
+    };
+    if ( $self->{ignore_missing_table} and ( $@ or @err ) and grep { $_ =~ $enoentrx } ( @err, $@ ) )
     {
-        eval { $self->open_tables( $data, 0, 0 ) };
-        if ( $@ and $@ =~ m/no such (table|file)/i )
-        {
-            return ( -1, 0 );
-        }
+        $@ = '';
+        return ( -1, 0 );
     }
-    my ($eval) = $self->open_tables( $data, 0, 1 );
+
+    $self->do_err( $@ || $err[0] ) if ( $@ || @err );
 
     #    return undef unless $eval;
     return ( -1, 0 ) unless $eval;
@@ -237,7 +253,7 @@ sub DROP ($$$)
     my ($table) = $eval->table( $self->tables(0)->name() );
     $table->drop($data);
 
-    #use mylibs; zwarn $self->{f_stmt};
+    #use mylibs; zwarn $self->{sql_stmt};
     ( -1, 0 );
 }
 
@@ -246,13 +262,14 @@ sub INSERT ($$$)
     my ( $self, $data, $params ) = @_;
 
     my ( $eval, $all_cols ) = $self->open_tables( $data, 0, 1 );
-    return undef unless ($eval);
+    return unless ($eval);
 
     $eval->params($params);
     $self->verify_columns( $data, $eval, $all_cols ) if ( scalar( $self->columns() ) );
+    return if ( $self->{errstr} );
 
     my ($table) = $eval->table( $self->tables(0)->name() );
-    $table->seek( $data, 0, 2 );
+    $table->seek( $data, 0, 2 ) unless ( $table->capability('insert_new_row') );
 
     my ( $val, $col, $i, $k );
     my ($cNum) = scalar( $self->columns() );
@@ -291,7 +308,9 @@ sub INSERT ($$$)
                 }
                 $array->[ $table->column_num( $col->name() ) ] = $val;
             }
-            $table->push_row( $data, $array );
+            $table->capability('insert_new_row')
+              ? $table->insert_new_row( $data, $array )
+              : $table->push_row( $data, $array );
         }
     }
     else
@@ -308,44 +327,61 @@ sub DELETE ($$$)
     return unless $eval;
     $eval->params($params);
     $self->verify_columns( $data, $eval, $all_cols );
-    my ($table)    = $eval->table( $self->tables(0)->name() );
+    return if ( $self->{errstr} );
+    my $tname      = $self->tables(0)->name();
+    my ($table)    = $eval->table($tname);
     my ($affected) = 0;
     my ( @rows, $array );
 
-    if ( $table->can('delete_one_row') )
-    {
-        while ( my $array = $table->fetch_row($data) )
-        {
-            if ( $self->eval_where( $eval, '', $array ) )
-            {
-                ++$affected;
-                $array = $self->{fetched_value} if ( $self->{fetched_from_key} );
-                $table->delete_one_row( $data, $array );
-                return ( $affected, 0 ) if ( $self->{fetched_from_key} );
-            }
-        }
-
-        return ( $affected, 0 );
-    }
-
     while ( $array = $table->fetch_row($data) )
     {
-        if ( $self->eval_where( $eval, '', $array ) )
+        if ( $self->eval_where( $eval, $tname, $array ) )
         {
             ++$affected;
+            if ( $table->capability('rowwise_delete') and $table->capability('inplace_delete') )
+            {
+                if ( $table->capability('delete_one_row') )
+                {
+                    $table->delete_one_row( $data, $array );
+                }
+                elsif ( $table->capability('delete_current_row') )
+                {
+                    $table->delete_current_row( $data, $array );
+                }
+            }
+            elsif ( $table->capability('rowwise_delete') )
+            {
+                push( @rows, $array );
+            }
+
+            next;
+        }
+
+        push( @rows, $array ) unless ( $table->capability('rowwise_delete') );
+    }
+
+    if (@rows)
+    {
+        if ( $table->capability('rowwise_delete') )    # @rows is empty in case of inplace_delete capability
+        {
+            foreach my $array (@rows)
+            {
+                $table->delete_one_row( $data, $array );
+            }
         }
         else
         {
-            push( @rows, $array );
+            # rewrite table without deleted elements
+            $table->seek( $data, 0, 0 );
+            foreach my $array (@rows)
+            {
+                $table->push_row( $data, $array );
+            }
+            $table->truncate($data);
         }
     }
-    $table->seek( $data, 0, 0 );
-    foreach $array (@rows)
-    {
-        $table->push_row( $data, $array );
-    }
-    $table->truncate($data);
-    ( $affected, 0 );
+
+    return ( $affected, 0 );
 }
 
 sub UPDATE ($$$)
@@ -360,6 +396,7 @@ sub UPDATE ($$$)
     $self->{params} ||= $params;
     $eval->params($params);
     $self->verify_columns( $data, $eval, $all_cols );
+    return if ( $self->{errstr} );
 
     my $tname      = $self->tables(0)->name();
     my ($table)    = $eval->table($tname);
@@ -368,20 +405,17 @@ sub UPDATE ($$$)
 
     while ( my $array = $table->fetch_row($data) )
     {
+        my $originalValues;
         if ( $self->eval_where( $eval, $tname, $array ) )
         {
             my $valpos = 0;
-            my $originalValues;
-            if ( not( $self->{fetched_from_key} )
-                 and $table->can('update_specific_row') )
+            if ( $table->capability('update_specific_row') )
             {
                 $originalValues = clone($array);
-                $array          = $self->{fetched_value};
             }
 
             for ( my $i = 0; $i < $self->columns(); $i++ )
             {
-                my $col = $self->columns($i);
                 my $val = $self->row_values( 0, $i );
                 if ( defined( _INSTANCE( $val, 'SQL::Statement::Param' ) ) )
                 {
@@ -405,44 +439,65 @@ sub UPDATE ($$$)
                     return $self->do_err('Internal error: Unexpected column type');
                 }
 
+                my $col = $self->columns($i);
                 $array->[ $table->column_num( $col->name() ) ] = $val;
             }
 
             ++$affected;
-
-            # Martin Fabiani <martin@fabiani.net>:
-            # the following block is the most important enhancement to SQL::Statement::UPDATE
-            if ( !$self->{fetched_from_key} )
+            if ( $table->capability('rowwise_update') and $table->capability('inplace_update') )
             {
-                if ( $table->can('update_specific_row') )
+                # Martin Fabiani <martin@fabiani.net>:
+                # the following block is the most important enhancement to SQL::Statement::UPDATE
+                if ( $table->capability('update_specific_row') )
                 {
                     $table->update_specific_row( $data, $array, $originalValues );
-                    next;
                 }
-                elsif ( $table->can('update_one_row') )
+                elsif ( $table->capability('update_one_row') )
                 {
+                    # NOTE: this prevents from updating index keys
                     $table->update_one_row( $data, $array );
-                    next;
+                }
+                elsif ( $table->capability('update_current_row') )
+                {
+                    $table->update_current_row( $data, $array );
                 }
             }
-            else
+            elsif ( $table->capability('rowwise_update') )
             {
-                $table->update_one_row( $data, $array );
-                return ( $affected, 0 );
+                push( @rows, $array ) unless ( $table->capability('update_specific_row') );
+                push( @rows, [ $array, $originalValues ] ) if ( $table->capability('update_specific_row') );
             }
         }
 
-        push( @rows, $array ) unless ( $table->can('update_one_row') || $table->can('update_specific_row') );
+        push( @rows, $array ) unless ( $table->capability('rowwise_update') );
     }
 
-    unless ( $table->can('update_one_row') || $table->can('update_specific_row') )
+    if (@rows)
     {
-        $table->seek( $data, 0, 0 );
-        foreach my $array (@rows)
+        if ( $table->capability('rowwise_update') )    # @rows is empty in case of inplace_update capability
         {
-            $table->push_row( $data, $array );
+            foreach my $array (@rows)
+            {
+                if ( $table->capability('update_specific_row') )
+                {
+                    $table->update_specific_row( $data, $array->[0], $array->[1] );
+                }
+                elsif ( $table->capability('update_one_row') )
+                {
+                    $table->update_one_row( $data, $array );
+                }
+            }
+        }
+        else
+        {
+            # rewrite table with updated elements
+            $table->seek( $data, 0, 0 );
+            foreach my $array (@rows)
+            {
+                $table->push_row( $data, $array );
+            }
+            $table->truncate($data);
         }
-        $table->truncate($data);
     }
 
     return ( $affected, 0 );
@@ -541,6 +596,7 @@ sub JOIN
     return undef unless $eval;
     $eval->params($params);
     $self->verify_columns( $data, $eval, $all_cols );
+    return if ( $self->{errstr} );
     if (     $self->{join}->{keycols}
          and $self->{join}->{table_order}
          and ( scalar( @{ $self->{join}->{table_order} } ) == 0 ) )
@@ -826,6 +882,7 @@ sub SELECT($$)
         return unless $eval;
         $eval->params($params);
         $self->verify_columns( $data, $eval, $all_cols );
+        return if ( $self->{errstr} );
         $tableName = $self->tables(0)->name();
         $table     = $eval->table($tableName);
     }
@@ -956,16 +1013,13 @@ sub SELECT($$)
         {
             next if ( defined($limit_count) and ( $row_count++ < $offset ) );
             $limit_count++ if ( defined($limit_count) );
-            $array = $self->{fetched_value} if ( $self->{fetched_from_key} );
 
             my @row = map { $_->value($e) } $self->columns();
             push( @{$rows}, \@row );
 
-            # We quit here if its a primary key search
-            # or if there's a limit clause without order clause
+            # We quit here if there's a limit clause without order clause
             # and the limit has been reached
-            #
-            if ( $self->{fetched_from_key} || ( defined($limit_count) && ( $limit_count >= $self->limit() ) ) )
+            if ( defined($limit_count) && ( $limit_count >= $self->limit() ) )
             {
                 return ( scalar( @{$rows} ), $numFields, $rows );
             }
@@ -975,9 +1029,10 @@ sub SELECT($$)
     if ( $self->distinct() )
     {
         my %seen;
-        @{$rows} = map
-        {
-            $seen{ join( "\0", ( map { defined($_) ? $_ : '' } @{$_} ) ) }++ ? () : $_
+        @{$rows} = map {
+            $seen{ join( "\0", ( map { defined($_) ? $_ : '' } @{$_} ) ) }++
+              ? ()
+              : $_
         } @{$rows};
     }
 
@@ -1078,6 +1133,8 @@ sub fetch
     return $row;
 }
 
+sub open_table ($$$$$) { croak "Abstract method " . ref( $_[0] ) . "::open_table called" }
+
 sub open_tables
 {
     my ( $self, $data, $createMode, $lockMode ) = @_;
@@ -1110,27 +1167,20 @@ sub open_tables
         elsif ( $data->{Database}->{sql_ram_tables}->{$name} )
         {
             $t->{$name} = $data->{Database}->{sql_ram_tables}->{$name};
-            $t->{$name}->{index} = 0;
+            $t->{$name}->seek( $data, 0, 0 );
             $t->{$name}->init_table( $data, $name, $createMode, $lockMode ) if ( $t->{$name}->can('init_table') );
         }
-        elsif ( $self->{is_ram_table} or !( $self->can('open_table') ) )
+        elsif ( $self->{is_ram_table} )
         {
-            $t->{$name} = $data->{Database}->{sql_ram_tables}->{$name} = SQL::Statement::RAM->new( $name, [], [] );
+            $t->{$name} = $data->{Database}->{sql_ram_tables}->{$name} =
+              SQL::Statement::RAM::Table->new( $name, [], [] );
         }
         else
         {
             undef $@;
             eval {
                 my $open_name = $self->{org_table_names}->[$count];
-                if ( $caller && $caller =~ m/^DBD::AnyData/ )
-                {
-                    $caller .= '::Statement' unless ( $caller =~ m/::Statement/ );
-                    $t->{$name} = $caller->open_table( $data, $open_name, $createMode, $lockMode );
-                }
-                else
-                {
-                    $t->{$name} = $self->open_table( $data, $open_name, $createMode, $lockMode );
-                }
+		$t->{$name} = $self->open_table( $data, $open_name, $createMode, $lockMode );
             };
             my $err = $t->{$name}->{errstr};
             return $self->do_err($err) if ($err);
@@ -1367,20 +1417,24 @@ sub verify_expand_column
 {
     my ( $self, $c, $i, $usr_cols, $is_duplicate, $col_exists ) = @_;
 
-    my ( $table, $col );
+    my ( $table, $col, $col_obj );
     if ( $c =~ m/(\S+)\.(\S+)/ )
     {
         $table = $1;
         $col   = $2;
     }
-    else
+    elsif ( ++${$i} >= 0 )
     {
-        ++${$i};
-        ( $table, $col ) = ( $usr_cols->[ ${$i} ]->{table}, $usr_cols->[ ${$i} ]->{name} ) if ( ${$i} >= 0 );
+        $col_obj = $usr_cols->[ ${$i} ];
+        ( $table, $col ) = ( $col_obj->{table}, $col_obj->{name} );
     }
     return unless ($col);
 
-    unless ( defined($table) )
+    my $is_column =
+      ( defined( _INSTANCE( $col_obj, 'SQL::Statement::Util::Column' ) )
+        and ( $col_obj->{coldef}->{type} eq 'column' ) ) ? 1 : 0;
+
+    unless ( $is_column and defined($table) )
     {
         ( $table, undef ) = $self->full_qualified_column_name($col);
     }
@@ -1390,23 +1444,22 @@ sub verify_expand_column
         $table = $table->name();
     }
 
-    my $col_obj = $self->{computed_column}->{$c};
-    if ( !$table and !$col_obj )
+    if ( $is_column and !$table )
     {
         return $self->do_err("Ambiguous column name '$c'") if ( $is_duplicate->{$c} );
+        return $self->do_err("No such column '$col'");
         $col = $c;
     }
-    elsif ( !$col_obj )
+    elsif ($is_column)
     {
         my $is_user_def = 1 if ( $self->{opts}->{function_defs}->{$col} );
         return $self->do_err("No such column '$table.$col'")
           unless (    $col_exists->{"$table.$col"}
                    or $col_exists->{ "\L$table." . $col }
                    or $is_user_def );
-
     }
 
-    return ( $table, $col );
+    return ( $table, $col ) if ($is_column);
 }
 
 sub verify_columns
@@ -1806,11 +1859,19 @@ sub get_user_func_table
     # my $tempTable = SQL::Statement::TempTable->new(
     #     $name, $col_names, $col_names, $data_aryref
     # );
-    my $tempTable = SQL::Statement::RAM->new( $name, $col_names, $data_aryref );
+    my $tempTable = SQL::Statement::RAM::Table->new( $name, $col_names, $data_aryref );
     $tempTable->{all_cols} ||= $col_names;
     return $tempTable;
 }
 
+sub capability($)
+{
+    my ( $self, $capname ) = @_;
+    return $self->{capabilities}->{$capname} if ( defined( $self->{capabilities}->{$capname} ) );
+
+    return;
+}
+
 sub DESTROY
 {
     my $self = $_[0];
@@ -1823,11 +1884,8 @@ sub DESTROY
     undef $self->{col_obj};
     undef $self->{column_names};
     undef $self->{columns};
-    undef $self->{computed_column};
     undef $self->{cur_table};
     undef $self->{data};
-    undef $self->{fetched_value};
-    undef $self->{fetched_from_key};
     undef $self->{group_by};
     undef $self->{has_OR};
     undef $self->{join};
@@ -2035,9 +2093,13 @@ sub getAffectedResult    # (\@)
 package SQL::Statement::TempTable;
 
 use vars qw(@ISA);
-require SQL::Eval;
 
-@ISA = qw(SQL::Eval::Table);
+BEGIN
+{
+    require SQL::Eval;
+
+    @SQL::Statement::TempTable::ISA = qw(SQL::Eval::Table);
+}
 
 sub new
 {
@@ -2058,7 +2120,7 @@ sub new
                  rowpos     => 0,
                  maxrow     => scalar @$table
                };
-    return bless $self, $class;
+    return $class->SUPER::new($self);
 }
 
 sub is_shared($) { $_[0]->{is_shared}->{ $_[1] }; }
@@ -2068,14 +2130,14 @@ sub column_num($)
 {
     my ( $s, $col ) = @_;
     my $new_col = $s->{col_nums}->{$col};
-    if ( !defined $new_col )
+    unless ( defined($new_col) )
     {
-        my @tmp = split '~', $col;
-        return undef unless ( 2 == scalar(@tmp) );
+        my @tmp = split( '~', $col );
+        return unless ( 2 == scalar(@tmp) );
         $new_col = lc( $tmp[0] ) . '~' . $tmp[1];
         $new_col = $s->{col_nums}->{$new_col};
     }
-    $new_col;
+    return $new_col;
 }
 
 sub fetch_row()
@@ -2154,15 +2216,16 @@ SQL::Statement - SQL parsing and processing engine
 The SQL::Statement module implements a pure Perl SQL parsing and execution
 engine. While it by no means implements full ANSI standard, it does support
 many features including column and table aliases, built-in and user-defined
-functions, implicit and explicit joins, complexly nested search conditions,
+functions, implicit and explicit joins, complex nested search conditions,
 and other features.
 
-SQL::Statement is a small embeddable Database Management System (DBMS), this
-means that it provides all of the services of a simple DBMS except that
-instead of a persistent storage mechanism, it has two things: 1) an in-memory
-storage mechanism that allows you to prepare, execute, and fetch from SQL
-statements using temporary tables and 2) a set of software sockets where
-any author can plug in any storage mechanism.
+SQL::Statement is a small embeddable Database Management System
+(DBMS). This means that it provides all of the services of a simple
+DBMS except that instead of a persistent storage mechanism, it has two
+things: 1) an in-memory storage mechanism that allows you to prepare,
+execute, and fetch from SQL statements using temporary tables and 2) a
+set of software sockets where any author can plug in any storage
+mechanism.
 
 There are three main uses for SQL::Statement. One or another (hopefully not
 all) may be irrelevant for your needs: 1) to access and manipulate data in
@@ -2171,19 +2234,21 @@ CSV, XML, and other formats 2) to build your own DBD for a new data source
 
 =head1 INSTALLATION
 
-There are no prerequisites for using this as a standalone parser. If you want
-to access persistent stored data, you either need to write a subclass or use
-one of the DBI DBD drivers.  You can install this module using CPAN.pm,
-CPANPLUS.pm, PPM, apt-get, or other packaging tools.  Or you can download the
-tar.gz file form CPAN and use the standard perl mantra:
+There are no prerequisites for using this as a standalone parser. If
+you want to access persistent stored data, you either need to write a
+subclass or use one of the DBI DBD drivers.  You can install this
+module using CPAN.pm, CPANPLUS.pm, PPM, apt-get, or other packaging
+tools or you can download the tar.gz file from CPAN and use the
+standard perl mantra:
 
   perl Makefile.PL
   make
   make test
   make install
 
-It works fine on all platforms it's been tested on. On Windows, you can use
-ppm or with the mantra use nmake, dmake, or make depending on which is available.
+It works fine on all platforms it has been tested on. On Windows, you
+can use ppm or with the mantra use nmake, dmake, or make depending on
+which is available.
 
 =head1 USAGE
 
@@ -2198,9 +2263,9 @@ HTML and many other kinds of text files, to Excel Spreadsheets, to BerkeleyDB
 and other DBM formats, and to non-traditional data sources like on-the-fly
 Amazon searches.
 
-If you're interested in accessing and manipulating persistent data, you may
+If you are interested in accessing and manipulating persistent data, you may
 not really want to use SQL::Statement directly, but use L<DBI> along with
-one of the DBDs mentioned above instead. You'll be using SQL::Statement, but
+one of the DBDs mentioned above instead. You will be using SQL::Statement, but
 under the hood of the DBD. See L<http://dbi.perl.org> for help with DBI and
 see L<SQL::Statement::Syntax> for a description of the SQL syntax that
 SQL::Statement provides for these modules and see the documentation for
@@ -2208,9 +2273,9 @@ whichever DBD you are using for additional details.
 
 =head2 How can I use it to parse and examine the structure of SQL statements?
 
-SQL::Statement can be used stand-alone (without a subclass, without DBI) to
-parse and examine the structure of SQL statements.
-See L<SQL::Statement::Structure> for details.
+SQL::Statement can be used stand-alone (without a subclass and without
+DBI) to parse and examine the structure of SQL statements.  See
+L<SQL::Statement::Structure> for details.
 
 =head2 How can I use it to embed a SQL engine in a DBD or other module?
 
@@ -2236,10 +2301,119 @@ directions. If you want to join the development team, or just hear more
 about the development, write Jeff (<jzuckerATcpan.org>) or Jens
 (<rehsackATcpan.org>) a note.
 
-=begin undocumented
-
 =head1 METHODS
 
+The following methods can or must be overridden by derived classes.
+
+=head2 capability
+
+  $has_capability = $h->capability('capability_name');
+
+Returns a true value if the specified capability is available.
+
+Currently no capabilities are defined and this is a placeholder for
+future use. It is envisioned it will be used like C<<
+SQL::Eval::Table::capability >>.
+
+=head2 open_table
+
+The C<< open_table >> method must be overridden by derived classes to provide
+the capability of opening data tables. This is a necessity.
+
+Arguments given to open_table call:
+
+=over 4
+
+=item C<< $data >>
+
+The database memo parameter. See L</execute>.
+
+=item C<< $table >>
+
+The name of the table to open as parsed from SQL statement.
+
+=item C<< $createMode >>
+
+A flag indicating the mode (C<< CREATE TABLE ... >>) the table should
+be opened with. Set to a true value in create mode.
+
+=item C<< $lockMode >>
+
+A flag indicating whether the table should be opened for writing (any
+other than C<< SELECT ... >>).  Set to a true value if the table is to
+be opened for write access.
+
+=back
+
+The following methods are required to use SQL::Statement in a DBD (for
+example).
+
+=head2 new
+
+Instantiates a new SQL::Statement object.
+
+Arguments:
+
+=over 4
+
+=item C<< $sql >>
+
+The SQL statement for later actions.
+
+=item C<< $parser >>
+
+An instance of a L<SQL::Parser> object or flags for it's instantiation.
+If omitted, default flags are used.
+
+=back
+
+When the basic initialization is completed,
+C<< $self->prepare($sql, $parser) >> is invoked.
+
+=head2 prepare
+
+Prepares SQL::Statement to execute a SQL statement.
+
+Arguments:
+
+=over 4
+
+=item C<< $sql >>
+
+The SQL statement to parse and prepare.
+
+=item C<< $parser >>
+
+Instance of a L<SQL::Parser> object to parse the provided SQL statement.
+
+=back
+
+=head2 execute
+
+Executes a prepared statement.
+
+Arguments:
+
+=over 4
+
+=item C<< $data >>
+
+Memo field passed through to calls of the instantiated C<< $table >>
+objects or C<< open_table >> calls. In C<< CREATE >> with subquery,
+C<< $data->{Database} >> must be a DBI database handle object.
+
+=item C<< $params >>
+
+Bound params via DBI ...
+
+=back
+
+=head2 errstr
+
+Gives the error string of the last error, if any.
+
+=begin undocumented
+
 =head2 _anycmp
 
 =head2 buildColumnObjects
@@ -2260,12 +2434,8 @@ about the development, write Jeff (<jzuckerATcpan.org>) or Jens
 
 =head2 do_err
 
-=head2 errstr
-
 =head2 eval_where
 
-=head2 execute
-
 =head2 fetch
 
 =head2 find_join_columns
@@ -2290,8 +2460,6 @@ about the development, write Jeff (<jzuckerATcpan.org>) or Jens
 
 =head2 params
 
-=head2 prepare
-
 =head2 row_values
 
 =head2 run_functions
@@ -2310,36 +2478,64 @@ about the development, write Jeff (<jzuckerATcpan.org>) or Jens
 
 =end undocumented
 
-=head1 Where can I go for more help?
+=head1 SUPPORT
+
+You can find documentation for this module with the perldoc command.
+
+    perldoc SQL::Statement
+
+You can also look for information at:
+
+=over 4
+
+=item * RT: CPAN's request tracker
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=SQL-Statement>
+
+=item * AnnoCPAN: Annotated CPAN documentation
+
+L<http://annocpan.org/dist/SQL-Statement>
+
+=item * CPAN Ratings
+
+L<http://cpanratings.perl.org/s/SQL-Statement>
+
+=item * CPAN Search
+
+L<http://search.cpan.org/dist/SQL-Statement/>
+
+=back
+
+=head2 Where can I go for help?
 
 For questions about installation or usage, please ask on the
-dbi-users@perl.org mailing list or post a question on PerlMonks
-(L<http://www.perlmonks.org/>, where Jeff is known as jZed).
-Jens didn't visit PerlMonks on a regular basis.
+dbi-users@perl.org mailing list (see http://dbi.perl.org) or post a
+question on PerlMonks (L<http://www.perlmonks.org/>, where Jeff is
+known as jZed).  Jens does not visit PerlMonks on a regular basis.
 
-If you have a bug report, a patch, a suggestion, please open
-a new report ticket at CPAN, if there isn't already a one for
-the issue you want to report. Of course, you can mail any of the
-module maintainers, but you'll be sure that you're report will
-not be "forgotten" or reach an (temporarily) inactive maintainer.
-Report tickets should contain a detailed description of the
-bug or enhancement request you want to report and at least an
-easy to verify and use test to reproduce the issue and verify the
-applied fix. Patches are always welcome, too.
+If you have a bug report, a patch or a suggestion, please open a new
+report ticket at CPAN (but please check previous reports first in case
+your issue has already been addressed). You can mail any of the module
+maintainers, but you are more assured of an answer by posting to
+the dbi-users list or reporting the issue in RT.
+
+Report tickets should contain a detailed description of the bug or
+enhancement request and at least an easily verifiable way of
+reproducing the issue or fix. Patches are always welcome, too.
 
 =head2 Where can I go for help with a concrete version?
 
-Bugs and feature requests are accepted against latest version
+Bugs and feature requests are accepted against the latest version
 only. To get patches for earlier versions, you need to get an
-agreement with a developer of your choice - who might or might
-not report the issue and a suggested fix upstream (depends on
-the license you've chosen).
+agreement with a developer of your choice - who may or not report the
+issue and a suggested fix upstream (depends on the license you have
+chosen).
 
 =head2 Business support and maintenance
 
-For business support you can contact Jens via it's CPAN email
+For business support you can contact Jens via his CPAN email
 address rehsackATcpan.org. Please keep in mind that business
-support is neither available for free nor you're eligible to
+support is neither available for free nor are you eligible to
 receive any support based on the license distributed with this
 package.
 
@@ -2347,15 +2543,17 @@ package.
 
 Jochen Wiedmann created the original module as an XS (C) extension in 1998.
 Jeff Zucker took over the maintenance in 2001 and rewrote all of the C
-portions in perl and began extending the SQL support.  More recently Ilya
+portions in Perl and began extending the SQL support.  More recently Ilya
 Sterin provided help with SQL::Parser, Tim Bunce provided both general and
 specific support, Dan Wright and Dean Arnold have contributed extensively
 to the code, and dozens of people from around the world have submitted
 patches, bug reports, and suggestions.
-In 2008 Jens Rehsack took over maintenance the extended module from Jeff.
-Together with H.Merijn Brand (who has taken DBD::CSV), Detlef Wartke and
-Volker Schubbert (especially between 1.16 developer versions until 1.22)
-and all submitters of bug reports via RT a lot of issues became fixed.
+
+In 2008 Jens Rehsack took over the maintenance of the extended module
+from Jeff.  Together with H.Merijn Brand (who has taken DBD::CSV),
+Detlef Wartke and Volker Schubbert (especially between 1.16 developer
+versions until 1.22) and all submitters of bug reports via RT a lot of
+issues have been fixed.
 
 Thanks to all!
 
@@ -2368,40 +2566,42 @@ with your own modules, feel free to contact Jeff or Jens.
 
 =item *
 
-currently we treat NULL and '' as the same in AnyData/CSV mode - eventually fix
+Currently we treat NULL and '' as the same in AnyData/CSV mode -
+eventually fix.
 
 =item *
 
-No nested C-style comments allowed as SQL99 says
+No nested C-style comments allowed as SQL99 says.
 
 =item *
 
-There're some issues regarding combining outer joins with where clauses.
+There are some issues regarding combining outer joins with where
+clauses.
 
 =item *
 
-Aggregate functions can't be used in where clause.
+Aggregate functions cannot be used in where clause.
 
 =item *
 
-Some SQL commands/features are not supported (most of them can't by design),
-as C<LOCK TABLE>, using indices, sub-selects etc.
+Some SQL commands/features are not supported (most of them cannot by
+design), as C<LOCK TABLE>, using indices, sub-selects etc.
 
-Currently the statement to missing features is: I plan to create a
+Currently the statement for missing features is: I plan to create a
 SQL::Statement v2.00 based on a pure Backus-Naur-Form parser and a
 fully object oriented command pattern based engine implementation.
-When the time is available, I will do it. As long, mostly bugs will
-be fixed or other Perl modules under my maintainership will receive
-my time. Feature which can be added without deeper design changes
-might get applied earlier - especially when their addition allows
-studying effective ways to implement the feature in upcoming 2.00.
+When the time is available, I will do it. Until then bugs will be
+fixed or other Perl modules under my maintainership will receive my
+time. Features which can be added without deep design changes might be
+applied earlier - especially when their addition allows studying
+effective ways to implement the feature in upcoming 2.00.
 
 =item *
 
-Some people report, that SQL::Statement is getting slower since
-the XS parts are implemented in pure perl. This might be true, but on the
-other hand the amount of supported features have been increased dramatically
-as well as the support of the ANSI SQL 99 standard.
+Some people report that SQL::Statement is slower since the XS parts
+were implemented in pure Perl. This might be true, but on the other
+hand a large number of features have been added including support for
+ANSI SQL 99.
 
 For SQL::Statement 1.xx it's not planned to add new XS parts.
 
@@ -2410,8 +2610,8 @@ For SQL::Statement 1.xx it's not planned to add new XS parts.
 Wildcards are expanded to lower cased identifiers. This might confuse
 some people, but it was easier to implement.
 
-The warning from L<DBI>, never trust on case sensetiveness of returned column
-names should be read more often. If you need to rely on identifiers, always
+The warning in L<DBI> to never trust the case of returned column names
+should be read more often. If you need to rely on identifiers, always
 use C<sth-E<gt>{NAME_lc}> or C<sth-E<gt>{NAME_uc}> - never rely on
 C<sth-E<gt>{NAME}>:
 
@@ -2425,13 +2625,18 @@ See L<DBI/FetchHashKeyName> for more information.
 
 =back
 
-Patches to fix those bugs/limitations (or a grant to do it) would be very
-welcome. Please note, that these patches B<must> pass successful all tests
-of C<SQL::Statement>, L<DBD::File> and L<DBD::CSV> and must be a general
-improvement.
+Patches to fix bugs/limitations (or a grant to do it) would be
+very welcome. Please note, that any patches B<must> successfully pass
+all the C<SQL::Statement>, L<DBD::File> and L<DBD::CSV> tests and must
+be a general improvement.
 
 =head1 AUTHOR AND COPYRIGHT
 
+Jochen Wiedmann created the original module as an XS (C) extension in 1998.
+Jeff Zucker took over the maintenance in 2001 and rewrote all of the C
+portions in perl and began extending the SQL support. Since 2008, Jens
+Rehsack is the maintainer.
+
 Copyright (c) 2001,2005 by Jeff Zucker: jzuckerATcpan.org
 Copyright (c) 2007-2010 by Jens Rehsack: rehsackATcpan.org
 
